/*****************************************************************
**
**	@(#) rule.c	(c) Jul 2011 by hznet
**
**	Copyright (c) 2005 - 2010, Holger Zuleger HZnet. All rights reserved.
**	This software is open source.
**
**	Redistribution and use in source and binary forms, with or without
**	modification, are permitted provided that the following conditions
**	are met:
**
**	Redistribution of source code must retain the above copyright notice,
**	this list of conditions and the following disclaimer.
**
**	Redistribution's in binary form must reproduce the above copyright notice,
**	this list of conditions and the following disclaimer in the documentation
**	and/or other materials provided with the distribution.
**
**	Neither the name of Holger Zuleger HZnet nor the names of its contributors may
**	be used to endorse or promote products derived from this software without
**	specific prior written permission.
**
**	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
**	"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
**	TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
**	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
**	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
**	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
**	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
**	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
**	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
**	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
**	POSSIBILITY OF SUCH DAMAGE.
**
*****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>	/* assert() */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#define extern
# include "rule.h"
#undef extern
# include "spmake.h"
# include "debug.h"


static	rule_t	rules[MAXRULES];
int	nrules;

int	readrulefile (FILE *fp)
{
	char	buf[255+1];
	char	*name;
	char	*p;
	int	single;
	int	lnr;

	assert ( fp != NULL );

	lnr = 0;
	while ( fgets (buf, sizeof (buf), fp) )
	{
		lnr++;

		p = buf;
		while ( isspace (*p) )
			p++;

			/* skip empty and comment lines */
		if ( *p == '\0' || *p == '#' || *p == ';' )
			continue;

		name = p;
		while ( *p && isalnum (*p) )
			p++;
		if ( *p == '\0' )
		{
			fprintf (stderr, "Illegal rule \'%s\' in line %d skipped\n", buf, lnr);
			continue;
		}
		*p++ = '\0';

		while ( isspace (*p) )
			p++;
		if ( *p != '1' && *p != '0' )
		{
			fprintf (stderr, "0/1 expected in line %d near '%s'\n", lnr, p);
			continue;
		}
		single = *p++ - '0';

		while ( *p && isalnum (*p) )
			p++;

		putrule (name, single, p);
	}

	return nrules;
}

int	putrule (const char *name, int single, const char *rule)
{
	char	*v[3];
	char	*dup;
	char	*p;
	int	i;

	if ( nrules >= MAXRULES )
	{
		fprintf (stderr, "Can't add rule %s to rule list: maximum number of rules (%d) reached\n", name, nrules);
		return 0;
	}

	assert ( name != NULL );
	assert ( rule != NULL );

	while ( *rule && isspace (*rule) )
		rule++;
	dup = strdup (rule);
	if ( dup == NULL )
	{
		fprintf (stderr, "out of memory\n");
		exit (1);
	}

	/* parse rule (Assumption: No colons in between the three rule parts)*/
	i = 0;
	p = dup;
	v[i++] = p;	/* first entry: source pattern */
	p = strchr (p, ':');
	if ( *p )
	{
		*p++ = '\0';
		v[i++] = p;	/* second entry: destination pattern */
	}
	p = strchr (p, ':');
	if ( *p )
	{
		*p++ = '\0';
		v[i++] = p;	/* third entry: command string */
	}

	if ( i != 3 )
	{
		fprintf (stderr, "error parsing rule %s: too %s elements\n", rule, i < 3 ? "less": "many");
		return 1;
	}

		/* remove end of line comment in command string */
	if ( (p = strrchr (v[2], '#')) != NULL )
		*p = '\0';

		/* remove white space at the end of command string */
	p = v[2] + strlen (v[2]) - 1;
	while ( p > v[2] && isspace (*p) )
		p--;
	*++p = '\0';

	rules[nrules].name = strdup (name);
	rules[nrules].single = single;

	rules[nrules].src = v[0];
	rules[nrules].target = v[1];
	rules[nrules].cmd = v[2];

	return ++nrules;
}

/*****************************************************************
**	getrule (name)
**	lookup rule 'name' in the list of rules.
*****************************************************************/
rule_t	*getrule (const char *name, int singletarget)
{
	int	i;

	assert ( name != NULL );

	for ( i = 0; i < nrules; i++ )
	{
		if ( rules[i].single == singletarget &&
				strcasecmp (name, rules[i].name) == 0 )
			return &rules[i];
	}

	return NULL;
}

/*****************************************************************
**	printrules (fp)
*****************************************************************/
int     printrules (FILE *fp)
{
	int	i;

	assert ( fp != NULL );
	assert ( nrules <= MAXRULES );

	for ( i = 0; i < nrules; i++ )
	{
		if ( verbose )
			fprintf (fp, "%2d ", i);
		fprintf (fp, "%-9s\t", rules[i].name);
		fprintf (fp, "%d\t", rules[i].single);
		fprintf (fp, "%s:%s:%s\n", rules[i].src, rules[i].target, rules[i].cmd);
	}

	return 0;
}

